
#include "jmp_score.h"
#include "DynamicCaller.h"
#include <iostream>
#include <string>
#include <sstream>
#include <vector>
#include <map>

#include <windows.h> 
#include <stdio.h> 
#include <strsafe.h>
#include <fstream>
#include <iostream>
#include <memory>

using std::cout;
using std::cin;
using std::cerr;
using std::endl;
using std::string;
using std::vector;

//class DynamicCaller : public Caller 
//public:


// From https://msdn.microsoft.com/en-us/library/windows/desktop/ms680582(v=vs.85).aspx
static void ErrorExit(LPTSTR lpszFunction)
{
	// Retrieve the system error message for the last-error code

	LPVOID lpMsgBuf;
	LPVOID lpDisplayBuf;
	DWORD dw = GetLastError();

	FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER |
		FORMAT_MESSAGE_FROM_SYSTEM |
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		dw,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR)&lpMsgBuf,
		0, NULL);

	// Display the error message and exit the process

	lpDisplayBuf = (LPVOID)LocalAlloc(LMEM_ZEROINIT,
		(lstrlen((LPCTSTR)lpMsgBuf) + lstrlen((LPCTSTR)lpszFunction) + 40) * sizeof(TCHAR));
	StringCchPrintf((LPTSTR)lpDisplayBuf,
		LocalSize(lpDisplayBuf) / sizeof(TCHAR),
		TEXT("%s failed with error %d: %s"),
		lpszFunction, dw, lpMsgBuf);
	std::wcerr << (LPCTSTR)lpDisplayBuf << endl;

	LocalFree(lpMsgBuf);
	LocalFree(lpDisplayBuf);
	ExitProcess(dw);
}

DynamicCaller::DynamicCaller(std::wstring& dllName) : scoreDll(nullptr), Caller() {
	if (dllName.empty()) {
		cerr << "Error: No DLL provided." << endl;
		cerr << "Please specify one with --dll <PATH>";
		return;
	}
	SetErrorMode(SEM_FAILCRITICALERRORS);
	scoreDll = LoadLibrary(dllName.c_str());
	if (scoreDll == nullptr) {
		cerr << "Error: Can't load DLL "; std::wcerr << dllName << endl;
		ErrorExit(TEXT("LoadLibrary"));
		return;
	}
	p_fillMetadataCounts = (PFMC)GetProcAddress(scoreDll, "fillMetadataCounts");
	p_fillModelMetadata = (PFMM)GetProcAddress(scoreDll, "fillModelMetadata");
	p_fillInputMetadata = (PFIM)GetProcAddress(scoreDll, "fillInputMetadata");
	p_fillOutputMetadata = (PFOM)GetProcAddress(scoreDll, "fillOutputMetadata");
	p_score = (PSCR)GetProcAddress(scoreDll, "score");
}

DynamicCaller::~DynamicCaller() {
	if (scoreDll != nullptr) {
		FreeLibrary(scoreDll);
	}
}

bool DynamicCaller::isValid() {
	return scoreDll != nullptr &&
		p_fillMetadataCounts != nullptr &&
		p_fillModelMetadata != nullptr &&
		p_fillInputMetadata != nullptr &&
		p_fillOutputMetadata != nullptr &&
		p_score != nullptr;
}

//Implements a wrapper around the JMP generated API suitable for dynamic linking
void DynamicCaller::fillMetadataCounts(COUNTS *mc) {
	(p_fillMetadataCounts)(mc);
}
void DynamicCaller::fillModelMetadata(MODELMD *mm) {
	(p_fillModelMetadata)(mm);
}
void DynamicCaller::fillInputMetadata(FIELDMD *fd) {
	(p_fillInputMetadata)(fd);
}
void DynamicCaller::fillOutputMetadata(FIELDMD *fd) {
	(p_fillOutputMetadata)(fd);
}
void DynamicCaller::score(PARM *indata, PARM *outdata) {
	(p_score)(indata, outdata);
}

