// DynCaller.cpp : Defines the entry point for the console application.
#include <memory>
#include "DynamicCaller.h"
#include "ModelTester.h"
#include "ArgReader.h"

int main(int argc, char *argv[]) {
	ArgReader reader(true); //dynamic
	if (reader.parse_args(argc, argv) > 0) 
		return 1;

#if 1
	DynamicCaller caller(reader.dllFileName);
#else
	StaticCaller caller;
#endif
	if (!caller.isValid()) {
		return 1;
	}
	std::unique_ptr<ModelTester> tester = nullptr; 
	if (reader.interactive) {
		tester = std::make_unique<ConsoleTester>(reader.verbose, caller);
	}
	else {
		tester = std::make_unique<FileTester>(reader.verbose, caller, reader.inputFileName, reader.outputFileName, reader.tolerance);
	}
	if (!tester->isValid()) {
		return 1;
	}
	int result = tester->test(reader.validate);
	reader.done();
	return result;
}

