#include <iostream>
#include <string>
#include <sstream>
#include <vector>
#include <map>

#include <windows.h> 
#include <stdio.h> 
#include <fstream>
#include <iostream>
#include <memory>
#include "ArgReader.h"
#include "Constants.h"

using std::cout;
using std::cin;
using std::cerr;
using std::endl;
using std::string;
using std::vector;

//class ArgReader 
//public:
void ArgReader::done() {
	if (pause || interactive) {
		cerr << "Press <ENTER> to exit." << endl;
		cin.get();
	}
}

void ArgReader::show_usage(std::string name) {
	std::cerr << "Usage: " << name << " <option(s)>" << endl
		<< "Options:" << endl
		<< "\t-h,--help          Show this help message" << endl
		<< "\t-v,--verbose       Prints additional information" << endl
		<< "\t-t,--validate      Validates calculation with values from input table" << endl
		<< "\t-p,--pause         Pause at the end to wait for user confirmation" << endl
		<< "\t-x,--interactive   Reads input variables from console, one value at a time" << endl
		<< "\t-i,--input <FILE>  Input file (scoring input and validation columns)" << endl
		<< "\t-o,--ouput <FILE>  Output file (scoring output and validation results)" << endl
		<< "\t-e,--tolerance <N> Tolerance percentage in numberic comparison (default: " << DEFAULT_TOLERANCE << ")" << endl;
	if (dynamic)
		std::cerr << "\t-d,--dll <FILE>    Scoring DLL file (required)" << endl;
	std::cerr << endl;
}

int ArgReader::read_arg(int &i, const int argc, char* argv[], const char *arg,
	const char* sform, const char *lform, std::wstring& target) {
	if (strcmp(arg, sform) == 0 || strcmp(arg, lform) == 0) {
		if (i + 1 < argc) {
			wchar_t wstr[256];
			size_t returnValue;
			mbstowcs_s(&returnValue, wstr, 256, argv[++i], _TRUNCATE);
			target = wstr;
			return -1;
		}
		else {
			std::cerr << "Error: Missing argument for " << lform << "!" << endl;
			show_usage(argv[0]);
			return 1;
		}
	}
	return 0;
}

bool ArgReader::parse_flag(const char * arg, const char* shortFlag, const char* longFlag, bool& flag) {
	if (strcmp(arg, shortFlag) == 0 || strcmp(arg, longFlag) == 0) {
		flag = true;
		return true;
	}
	return false;
}

int ArgReader::parse_args(int argc, char* argv[]) {
	if (argc < 3) {
		show_usage(argv[0]);
		return 1;
	}
	for (int i = 1; i < argc; ++i) {
		const char *arg = argv[i];
		int result = 0;
		bool dummy;
		if (parse_flag(arg, "-h", "--help", dummy)) {
			show_usage(argv[0]);
			return 0;
		}
		if (parse_flag(arg, "-x", "--interactive", interactive)) continue;
		if (parse_flag(arg, "-v", "--verbose", verbose)) continue;
		if (parse_flag(arg, "-t", "--validate", validate)) continue;
		if (parse_flag(arg, "-p", "--pause", validate)) continue;
		if (dynamic) {
			result = read_arg(i, argc, argv, arg, "-d", "--dll", dllFileName);
			if (result < 0) continue;
			if (result == 1) return 1;
		}
		result = read_arg(i, argc, argv, arg, "-i", "--input", inputFileName);
		if (result < 0) continue;
		if (result == 1) return 1;
		result = read_arg(i, argc, argv, arg, "-o", "--output", outputFileName);
		if (result < 0) continue;
		if (result == 1) return 1;
		result = read_arg(i, argc, argv, arg, "-e", "--tolerance", tolerance);
		if (result < 0) continue;
		if (result == 1) return 1;
		std::cerr << "Error: Unrecognized argument " << arg << endl;
		show_usage(argv[0]);
		return 1;
	}
	if (verbose) {
		wchar_t buffer[MAX_PATH];
		GetModuleFileName(NULL, buffer, MAX_PATH);
		std::wstring::size_type pos = std::wstring(buffer).find_last_of(L"\\/");
		std::wcerr << "Running from " << std::wstring(buffer).substr(0, pos) << endl;
	}
	return validate_options();
}

bool ArgReader::validate_options() {
	if (inputFileName.empty()) {
		if (validate) {
			std::cerr << "Error: Validate mode requires an input file." << endl;
			return 1;
		}
	}
	else {
		if (interactive) {
			std::cerr << "Error: Interactive mode not compatible with input file." << endl;
			return 1;
		}
	}
	if (!tolerance.empty()) {
		if (!validate) {
			std::cerr << "Warning: tolerance given, activating validate mode." << endl;
			validate = true;
		}

	}
	return	0;
}

ArgReader::ArgReader(bool isDynamic) : verbose(false), interactive(false), pause(false), validate(false), dynamic(isDynamic) {}
