#include "jmp_score.h"
#include "jmp_lib.h"
#include <iostream>
#include <string>
#include <sstream>
#include <vector>
#include <map>

#include <windows.h>
#include <stdio.h> 
#include <fstream>
#include <iostream>
#include <memory>
#include <math.h>
#include "ModelTester.h"
#include "Constants.h"

using std::cout;
using std::cin;
using std::cerr;
using std::endl;
using std::string;
using std::vector;

void ModelTester::setup() {
	loadMetadataCounts();
	loadModelMetadata();
	pInputMD = new FIELDMD[mc.numInputs];
	pOutputMD = new FIELDMD[mc.numOutputs];
	pInputs = new PARM[mc.numInputs];
	pOutputs = new PARM[mc.numOutputs];
	setupInputParms();
	setupOutputParms();
}

void ModelTester::loadMetadataCounts() {
	caller.fillMetadataCounts(&mc);
	if (verbose) {
		cerr << "= fillMetadataCounts ======================" << endl;
		cerr << "creatorLen:      " << mc.creatorLen << endl;
		cerr << "modelNameLen:    " << mc.modelNameLen << endl;
		cerr << "predictedLen:    " << mc.predictedLen << endl;
		cerr << "tableNameLen:    " << mc.tableNameLen << endl;
		cerr << "versionLen:      " << mc.versionLen << endl;
		cerr << "timestampLen:    " << mc.timestampLen << endl;
		cerr << "maxInputNameLen: " << mc.maxInputNameLen << endl;
		cerr << "maxOutputNameLen:" << mc.maxOutputNameLen << endl;
		cerr << "numInputs:       " << mc.numInputs << endl;
		cerr << "numOutputs:      " << mc.numOutputs << endl;
	}
}

void ModelTester::loadModelMetadata() {
	if (!verbose) // whole point of the fillModelMetadata() call is to provide 
		return;   // information to the user about the model being executed.

	MODELMD mm;
	char *pBuffer = new char[mc.creatorLen + mc.modelNameLen + mc.predictedLen + mc.tableNameLen + mc.versionLen + mc.timestampLen];
	mm.creator = pBuffer;
	mm.modelName = pBuffer + mc.creatorLen;
	mm.predicted = mm.modelName + mc.modelNameLen;
	mm.tableName = mm.predicted + mc.predictedLen;
	mm.version = mm.tableName + mc.tableNameLen;
	mm.timestamp = mm.version + mc.versionLen;

	caller.fillModelMetadata(&mm);
	cerr << "= fillModelMetadata ======================" << endl;
	cerr << "creator:   " << mm.creator << endl;
	cerr << "modelName: " << mm.modelName << endl;
	cerr << "predicted: " << mm.predicted << endl;
	cerr << "tableName: " << mm.tableName << endl;
	cerr << "Generated by JMP v" << mm.version << " on " << mm.timestamp << endl;
	delete[] pBuffer;
}

void ModelTester::cleanup() {
	for (int i = 0; i < mc.numInputs; i++) {
		delete[] pInputMD[i].name;
		if (pInputs[i].type == char_type)
			delete[] pInputs[i].data.str;
	}
	for (int i = 0; i < mc.numOutputs; i++) {
		delete[] pOutputMD[i].name;
		if (pOutputs[i].type == char_type)
			delete[] pOutputs[i].data.str;
	}
	delete[] pInputMD;
	delete[] pOutputMD;
	delete[] pInputs;
	delete[] pOutputs;
}

void ModelTester::setupInputParms() {
	for (int i = 0; i < mc.numInputs; i++) {
		pInputMD[i].name = new char[mc.maxInputNameLen];
	}
	caller.fillInputMetadata(pInputMD);
	for (int i = 0; i < mc.numInputs; i++) {
		setupParm(pInputMD[i], pInputs[i], true);
	}
}

void ModelTester::setupOutputParms() {
	for (int i = 0; i < mc.numOutputs; i++) {
		pOutputMD[i].name = new char[mc.maxOutputNameLen];
	}
	caller.fillOutputMetadata(pOutputMD);
	for (int i = 0; i < mc.numOutputs; i++) {
		setupParm(pOutputMD[i], pOutputs[i], false);
	}
}

// Initializes given output field.
// Allocates memory for string values as needed.
void ModelTester::setupParm(FIELDMD &fm, PARM& parm, bool isInput) {
	parm.type = fm.type;
	parm.datalen = fm.datalen;
	switch (fm.type) {
	case char_type:
		if (fm.datalen == 0 && isInput) { // datalen = 0 is always be the case
			fm.datalen = 256;             // for input variable metadata
		}
		parm.data.str = new char[fm.datalen]; break;
	case inum_type:
		parm.data.inum = 0; break;
	case fnum_type:
		parm.data.fnum = 0; break;
	}
}

void ModelTester::score() {
	caller.score(pInputs, pOutputs);
}

//public:
ModelTester::ModelTester(bool verboseValue, Caller& aCaller) :
	verbose(verboseValue), caller(aCaller),
	pInputMD(nullptr), pOutputMD(nullptr),
	pInputs(nullptr), pOutputs(nullptr) {
	setup();
}

ModelTester::~ModelTester() {
	cleanup();
}

bool ModelTester::isValid() {
	return true;
}

// class ConsoleTester : public ModelTester

void ConsoleTester::readInputParms() {
	cout << endl << "Reading input values: " << endl;
	for (int i = 0; i < mc.numInputs; i++) {
		readParm(pInputMD[i], pInputs[i]);
	}
}

// Prompt user for value for the given input field.
// Allocates memory for string values as needed.
void ConsoleTester::readParm(FIELDMD &fm, PARM& parm) {
	parm.type = fm.type;
	parm.datalen = fm.datalen;
	cout << "Enter value for field: " << fm.name << ", type ";
	switch (fm.type) {
	case char_type: cout << "string"; break;
	case inum_type: cout << "integer"; break;
	case fnum_type: cout << "double"; break;
	}
	cout << "\n>> ";
	size_t TWEET = 144 + 1;
	switch (fm.type) {
	case char_type:
		parm.data.str = new char[TWEET];
		cin.getline(parm.data.str, TWEET);
		break;
	case inum_type:
		while (true) {
			string input = "";
			std::getline(cin, input);
			// No check for missing - NaN not supported for integers
			// Converts from string to number safely.
			std::stringstream myStream(input);
			if (myStream >> parm.data.inum)
				break;
			cout << "Invalid integer, please try again" << endl;
			cout << "\n>> ";
		}
		break;
	case fnum_type:
		while (true) {
			string input = "";
			std::getline(cin, input);
			// Check for missing
			if (input.compare(".") == 0 || input.compare("") == 0) { // support both
				parm.data.fnum = MISSING;
				break;
			}
			// Converts from string to number safely.
			std::stringstream myStream(input);
			if (myStream >> parm.data.fnum)
				break;
			cout << "Invalid double, please try again" << endl;
			cout << "\n>> ";
		}
		break;
	}
}

void ConsoleTester::printOutputParms() {
	for (int i = 0; i < mc.numOutputs; i++) {
		printParm(pOutputMD[i], pOutputs[i]);
	}
}

void ConsoleTester::printParm(FIELDMD& fm, PARM& parm) {
	cout << "Output: " << fm.name << ", type ";
	switch (parm.type) {
	case char_type: cout << "string:" << parm.data.str; break;
	case inum_type: cout << "integer: " << parm.data.inum; break;
	case fnum_type: cout << "double: " << parm.data.fnum; break;
	}
	cout << endl;
}
//public:
ConsoleTester::ConsoleTester(bool verboseValue, Caller& aCaller) : ModelTester(verboseValue, aCaller) {}
ConsoleTester::~ConsoleTester() {}

int ConsoleTester::test(bool /*validate*/) {
	readInputParms();
	score();
	printOutputParms();
	return 0; //nothing to validate, return OK
}

//class FileTester : public ModelTester
void FileTester::tokenize(const std::string& str, std::vector<std::string>& tokens, const std::string& delimiters) {
	tokens.clear();
	string::size_type lastPos = str.find_first_not_of(delimiters, 0);
	string::size_type pos = str.find_first_of(delimiters, lastPos);
	string::size_type nEmpty = lastPos;
	string::size_type len = str.length();
	while (string::npos != pos || string::npos != lastPos) {
		for (int i = 0; i < int(nEmpty); i++) {
			tokens.push_back("");
		}
		tokens.push_back(str.substr(lastPos, pos - lastPos));
		//lastLastPos = pos;
		lastPos = str.find_first_not_of(delimiters, pos);
		nEmpty = lastPos - pos - 1;
		pos = str.find_first_of(delimiters, lastPos);
	}
}

bool FileTester::setupHeader(std::string& line, size_t& nCols, bool doValidate) {
	vector<string> tokens;
	tokenize(line, tokens);
	nCols = tokens.size();

	// create a map of header files to input fields
	for (size_t ti = 0; ti < tokens.size(); ++ti) {
		string& token = tokens[ti];
		bool found = false, foundAll = false;
		if (!foundAll) {
			for (int i = 0; i < mc.numInputs; ++i) {
				if (strcmp(pInputMD[i].name, token.c_str()) == 0) {
					if (verbose) cerr << "Found input field " << token << endl;
					inputFields.emplace(ti, &pInputMD[i]);
					inputs.emplace(ti, &pInputs[i]);
					found = true;
					foundAll = inputs.size() == mc.numInputs;
					break;
				}
			}
		}
		if (found) continue;
		for (int i = 0; i < mc.numOutputs; ++i) {
			if (strcmp(pOutputMD[i].name, token.c_str()) == 0) {
				if (verbose) cerr << "Found output field " << token << endl;
				outputFields.emplace(ti, &pOutputMD[i]);
				outputs.emplace(ti, &pOutputs[i]);
				validations.emplace(ti, 0);
				found = true;
				break;
			}
		}
		if (!found && verbose) {
			cerr << "Warning: unused column " << token << endl;
		}
	}
	if (inputFields.size() != mc.numInputs) {
		cerr << "Error: missing input fields!" << endl;
		return false;
	}
	if (doValidate && (outputFields.size() != mc.numOutputs)) {
		cerr << "Error: missing output fields (for validation)!" << endl;
		return false;
	}
	return true;
}

void FileTester::writeHeader(std::ostream& dest, bool doValidate) {
	bool first = true;
	for (int i = 0; i < mc.numOutputs; ++i) {
		if (first) first = false;
		else dest << ",";
		dest << pOutputMD[i].name;
	}
	if (doValidate) {
		std::map<size_t, FIELDMD*>::iterator it;
		for (it = outputFields.begin(); it != outputFields.end(); it++) {
			dest << "," << it->second->name << "-valid";
		}
	}
	dest << endl;
}

bool FileTester::setupInput(std::string& line, std::vector<string>& tokens, size_t expected) {
	tokenize(line, tokens);
	size_t nEmpty = expected - tokens.size();
	for (size_t i = 0; i < nEmpty; i++) {
		tokens.push_back("");
	}
	for (size_t i = 0; i < tokens.size(); ++i) {
		if (inputFields.find(i) != inputFields.end()) {
			auto token = tokens[i];
			FIELDMD* inputField = inputFields[i];
			PARM* input = inputs[i];
			if (!convertToken(token, *inputField, *input)) return false;
			continue;
		}
	}
	return true;
}

bool FileTester::convertToken(string& token, FIELDMD &fm, PARM& parm) {
	switch (fm.type) {
	case char_type: {
		size_t src_len = strlen(token.c_str()) + 1;
		if (fm.datalen < int(src_len)) {
			cerr << "Error: input value '" << "' larger than reserved size for field '" << fm.name << "'" << endl;
			return false;
		}
		memcpy(parm.data.str, token.c_str(), src_len);
	}
	break;
	case inum_type:
		if (token.compare("") == 0 || token.compare(".") == 0) {
			cerr << "Error: missing value not supported for integer field " << fm.name << endl;
			return false;
		}
		parm.data.inum = std::stoi(token); break;
	case fnum_type:
		if (token.compare("") == 0 || token.compare(".") == 0) {
			parm.data.fnum = MISSING;
		}
		else {
			parm.data.fnum = std::stod(token); 
		}
		break;
	default:
		cerr << "Error: Unsupported type for field " << fm.name << endl;
		return false;
	}
	return true;
}

void FileTester::writeOutput(std::ostream& dest, std::vector<string>& tokens, bool doValidate) {
	//copy data from pOutputs to line
	bool first = true;
	for (int i = 0; i < mc.numOutputs; ++i) {
		if (first) first = false;
		else dest << ",";
		convertOutput(dest, pOutputMD[i], pOutputs[i]);
	}
	if (doValidate) {
		totalLines++;
		if (validate(dest, tokens)) validLines++;
	}
	dest << endl;
}

void FileTester::convertOutput(std::ostream& dest, FIELDMD &fm, PARM& parm) {
	switch (fm.type) {
	case char_type:
		dest << parm.data.str; break;
	case inum_type:
		dest << parm.data.inum; break;
	case fnum_type:
		dest << parm.data.fnum;  break;
	}
}

bool FileTester::validate(std::ostream& dest, std::vector<string>& tokens) {
	//compare output results with matching input data
	bool validLine = true;
	for (size_t i = 0; i < tokens.size(); ++i) {
		if (outputFields.find(i) != outputFields.end()) {
			auto token = tokens[i];
			FIELDMD* outputField = outputFields[i];
			PARM* output = outputs[i];
			// convert value if needed, setup input
			bool isValid = validateOutput(dest, *outputField, *output, token);
			if (isValid) {
				validations[i]++;
			}
			else {
				validLine = false;
			}
			continue;
		}
	}
	return validLine;
}

bool FileTester::dbl_eq(double v1, double v2) {
	return (isnan(v1) && isnan(v2)) || (fabs(v1 - v2) <= max(fabs(v1), fabs(v2)) * toleranceValue);
}

bool FileTester::validateOutput(std::ostream& dest, FIELDMD &fm, PARM& parm, string& token) {
	bool isValid = false;
	switch (fm.type) {
	case char_type:
		isValid = strcmp(parm.data.str, token.c_str()) == 0; break;
	case inum_type:
		isValid = parm.data.inum == std::stoi(token); break;
	case fnum_type:
		double v = (token.compare("") == 0 || token.compare(".") == 0) ? MISSING : std::stod(token);
		isValid = dbl_eq(parm.data.fnum, v);  break;
	}
	dest << "," << isValid;
	return isValid;
}

//public:
FileTester::FileTester(bool verboseValue, Caller& aCaller, std::wstring& inputFileName, std::wstring& outputFileName, std::wstring& tolerance) :
	valid(true), totalLines(0), validLines(0),
	ModelTester(verboseValue, aCaller) {

	if (inputFileName.empty()) {
		if (verbose) cerr << "Reading from standard input." << endl;
		src = &cin;
	}
	else {
		srcFile.open(inputFileName);
		if (!srcFile.good()) {
			cerr << "Error: not able to open input file: "; std::wcerr << inputFileName << endl;
			valid = false;
		}
		else if (verbose) {
			cerr << "Reading file: "; std::wcerr << inputFileName << endl;
		}
		src = &srcFile;
	}
	if (outputFileName.empty()) {
		if (verbose) cerr << "Writing to standard output." << endl;
		dest = &cout;
	}
	else {
		destFile.open(outputFileName);
		if (!destFile.good()) {
			cerr << "Error: not able to open output file: "; std::wcerr << outputFileName << endl;
			valid = false;
		}
		else if (verbose) {
			cerr << "Writing to file: "; std::wcerr << outputFileName << endl;
		}
		dest = &destFile;
	}
	if (tolerance.empty())
		toleranceValue = DEFAULT_TOLERANCE;
	else {
		cerr << "Tolerance set to: "; std::wcerr << tolerance << endl;
		toleranceValue = std::stod(tolerance);
	}
}

FileTester::~FileTester() {
	if (srcFile.is_open()) srcFile.close();
	if (destFile.is_open()) destFile.close();
}

bool FileTester::isValid()  {
	return valid;
}

int FileTester::writeValidationTally() {
	cerr << "Validation results: " << endl;

	std::map<size_t, FIELDMD*>::iterator it;
	for (it = outputFields.begin(); it != outputFields.end(); it++) {
		size_t index = it->first;
		cerr << "\t" << it->second->name << ": "
			<< ((float)validations[index] / (float)totalLines) * 100 << "%" << endl;
	}
	cerr << endl;
	cerr << "\tTotal observations: " << totalLines << endl;
	cerr << "\tValid observations: " << validLines << endl;
	cerr << endl;
	return (totalLines == validLines) ? 0 : 2;
}

int FileTester::test(bool doValidate)  {
	std::string line;
	bool first = true;
	std::vector<string> tokens;
	std::vector<int> validations;
	size_t expected = 0;
	while (getline(*src, line)) {
		if (first) {
			if (!setupHeader(line, expected, doValidate)) return false;
			writeHeader(*dest, doValidate);
			first = false;
			continue;
		}
		if (!setupInput(line, tokens, expected))
			return 1;
		score();
		writeOutput(*dest, tokens, doValidate);
	}
	int result = 0;
	if (doValidate) {
		result = writeValidationTally();
	}
	return result;
}
