#pragma once

#include <iostream>
//#include <string>
//#include <sstream>
#include <vector>
#include <map>
//
//#include <windows.h>
//#include <stdio.h> 
#include <fstream>
//#include <memory>
//#include <math.h>

#include "jmp_score.h"
#include "ModelCaller.h"
#include "jmp_score.h"


class ModelTester
{
protected:
	bool verbose;
	COUNTS mc;
	FIELDMD* pInputMD;
	FIELDMD* pOutputMD;
	PARM* pInputs;
	PARM* pOutputs;
	Caller& caller;

	void setup();

	void loadMetadataCounts(); 
	void loadModelMetadata();
	void cleanup();
	void setupInputParms();
	void setupOutputParms();
	void setupParm(FIELDMD &fm, PARM& parm, bool isInput);
	void score();

public:
	ModelTester(bool verboseValue, Caller& aCaller);
	virtual ~ModelTester();
	virtual int test(bool validate) = 0;
	virtual bool isValid();
};

class ConsoleTester : public ModelTester
{
	void readInputParms();
	void readParm(FIELDMD &fm, PARM& parm);
	void printOutputParms();
	void printParm(FIELDMD& fm, PARM& parm);

public:
	ConsoleTester(bool verboseValue, Caller& aCaller);
	virtual ~ConsoleTester();
	int test(bool /*validate*/);
};

class FileTester : public ModelTester
{
	std::ifstream srcFile;
	std::ofstream destFile;
	std::istream* src;
	std::ostream* dest;

	std::map<size_t, FIELDMD*> inputFields;
	std::map<size_t, PARM*> inputs;
	std::map<size_t, FIELDMD*> outputFields;
	std::map<size_t, PARM*> outputs;
	std::map<size_t, int> validations;

	bool valid;
	int totalLines, validLines;
	double toleranceValue;

	void tokenize(const std::string& str, std::vector<std::string>& tokens, const std::string& delimiters = ",");
	bool setupHeader(std::string& line, size_t& nCols, bool doValidate);
	void writeHeader(std::ostream& dest, bool doValidate);
	bool setupInput(std::string& line, std::vector<std::string>& tokens, size_t expected);
	bool convertToken(std::string& token, FIELDMD &fm, PARM& parm);
	void writeOutput(std::ostream& dest, std::vector<std::string>& tokens, bool doValidate);
	void convertOutput(std::ostream& dest, FIELDMD &fm, PARM& parm);
	bool validate(std::ostream& dest, std::vector<std::string>& tokens);
	bool dbl_eq(double v1, double v2);
	bool validateOutput(std::ostream& dest, FIELDMD &fm, PARM& parm, std::string& token);
	int writeValidationTally();

public:
	FileTester(bool verboseValue, Caller& aCaller, std::wstring& inputFileName, std::wstring& outputFileName, 
		       std::wstring& tolerance);
	virtual ~FileTester();
	bool isValid() override;
	int test(bool validate) override;
};
