:: Build script for DLLs containing models in the C language generated by JMP 13+
:: Based on the Microsoft Visual Studio tool chain. 
:: For an example on how to score a data file using a scoring DLL, see score_dll.cmd


@echo off
setlocal

::Look for JMP Pro install directory in its default location
::Change the line below to reflect your local JMP Pro install
::Or set the corresponding environment variable
if "%JMP_PRO_INSTALLDIR%" == "" set JMP_PRO_INSTALLDIR=C:\Program Files\SAS\JMPPRO\13
if not exist "%JMP_PRO_INSTALLDIR%\jmp.exe" goto jmp_install
echo Found JMP Pro 13 at %JMP_PRO_INSTALLDIR%

::Find Visual Studio install dir from environment
::Uses the last VS installed version - watch out if you have multiple versions!!!
for /F "tokens=2 delims==" %%j in ('set ^| findstr COMNTOOLS') do set VSINSTALL_DIR=%%j

if "%VSINSTALL_DIR%" == "" goto env_usage
if not exist "%VSINSTALL_DIR%..\..\VC\vcvarsall.bat" goto env_usage
echo Found Visual Studio at %VSINSTALL_DIR%

if "%1"  == ""        goto usage
if /i %1 == x86       goto x86
if /i %1 == x64       goto x64
goto usage

:x86
call "%VSINSTALL_DIR%\..\..\VC\vcvarsall.bat" x86
goto :build_dll

:x64
call "%VSINSTALL_DIR%\..\..\VC\vcvarsall.bat" x64

:build_dll
if "%2" == "" goto missing_source
if /I "%~x2" NEQ ".c" goto wrong_extension

@echo Building %2 for %1

::For .c files captured from a JMP code editor using JSL, we have to first convert Mac line-endings to Unix.
::This is required to prevent compilation errors.
::For Windows, download the mac2unix tool from http://www.gammon.com.au/downloads/dlutilities.htm
::%~dp0/mac2unix %2

cl  /D "_DEBUG" /D "_WINDOWS" /D "_USRDLL" ^
    /D "JMP_SCORE_EXPORTS" /D "_WINDLL"    ^
    /I "%JMP_PRO_INSTALLDIR%\Scoring\C"  ^
    /I %~dp0\vs ^
    %~dp0\vs\dllmain.cpp %2 ^
    /link /DLL /OUT:"%~dp0..\out\%~n2_%1.dll"

if ERRORLEVEL 1 goto build_error
@echo Done.
goto end

:wrong_extension
@echo Incorrect extension for model file! 
@echo Expected ".c", found "%~x2" 
goto usage

:missing_source
@echo Missing model file name!
goto usage

:usage
@echo The correct usage is:
@echo     %~n0 ^[x86^|x64^] ^<MODEL^>.c
@echo   where ^[x86^|x64^] indicates the DLL target CPU architecture
@echo   and ^<MODEL^>.c is a C source code file
@echo     generated by JMP's Formula Depot 'Generate C Code' command.  
@echo.
@echo   Example: %~n0.cmd x64 ..\models\Iris_logistic.c
goto end

:env_usage
@echo ERROR: Visual Studio development environment not installed or configured.
@echo Please follow the instructions in https://msdn.microsoft.com/en-us/library/f2ccy3wt.aspx
goto end

:build_error
@echo ERROR: Compilation or environment error, DLL not created.
@echo Error level is: %ERRORLEVEL%
goto end

:jmp_install
@echo ERROR: JMP Pro 13 was not found at its usual location (C:\Program Files\SAS\JMPPRO\13)
@echo Please update this script to point to your local install directory
@echo or set the environment variable 'JMP_PRO_INSTALLDIR' to point to it.

:end
endlocal
