:: Example of how to run a scoring model from a DLL using the ModelRunner tool.
:: For an example on how to create a scoring DLL from a C source file, see build_dll.cmd


@echo off
setlocal

if "%1"  == ""        goto usage
if /i %1 == x86       goto x86
if /i %1 == x64       goto x64
goto usage

:x86
set MODEL_RUNNER=%~dp0\ModelRunner_x86.exe
goto :run

:x64
set MODEL_RUNNER=%~dp0\ModelRunner_x64.exe

:run
if "%2" == "" goto missing_dll
if /I "%~x2" NEQ ".dll" goto wrong_dll_extension
if "%3" == "" goto missing_data
if /I "%~x3" NEQ ".csv" goto wrong_data_extension

@echo Scoring %3 using  %2

%MODEL_RUNNER% -d %2 -v -i %3



@echo Done.
goto end

:wrong_dll_extension
@echo Incorrect extension for compiled model file! 
@echo Expected ".dll", found "%~x2" 
goto usage

:wrong_extension
@echo Incorrect extension for data file! 
@echo Expected ".csv", found "%~x3" 
goto usage

:missing_dll
@echo Missing compiled model file name!
goto usage

:missing_data
@echo Missing data file name!
goto usage

:usage
@echo The correct usage is:
@echo     %~n0 ^[x86^|x64^] ^<MODEL^>.dll ^<DATA^>.csv 
@echo   where ^[x86^|x64^] indicates the DLL target CPU architecture
@echo   and ^<MODEL^>.dll is a dynamic link library compiled from a C scoring code file
@echo     generated by JMP's Formula Depot 'Generate C Code' command
@echo   and ^<DATA^>.csv is a dataset with a compatible schema.
@echo.
@echo Example: %~n0.cmd x64 ..\out\Iris_logistic_x64.dll ..\data\iris.csv
goto end

:end
endlocal


