"""airline_delay.py"""
from __future__ import print_function
import sys
from pyspark import SparkContext, SparkConf
from pyspark.sql import SQLContext, Row
from pyspark.sql.types import *

def quiet_logs( sc ):            
    logger = sc._jvm.org.apache.log4j
    logger.LogManager.getLogger("org"). setLevel( logger.Level.ERROR )
    logger.LogManager.getLogger("akka").setLevel( logger.Level.ERROR )

def map_score(row, jmp_model):
    scored_row = {}
    input_row = row.asDict()
    jmp_model.score(input_row, scored_row)
    return Row(**scored_row)

if __name__ == "__main__":

    if len(sys.argv) == 1:
        print("Usage: airline_delay <input>", file=sys.stderr)
        exit(-1)

    input_file = sys.argv[1]
    output_file = None
    if len(sys.argv) >= 3:
        output_file = sys.argv[2]
    #import airlines_ArrDelay_neural2 as jmp_model
    import airlines_ArrDelay_FLSq as jmp_model

    # For local testing
    # sc = SparkContext("local", "Airline Delay")

    # For production (against a cluster). Master comes from CLI
    conf = SparkConf().setAppName("Airline Delay") #.setMaster(master)
    sc = SparkContext(conf=conf)
    quiet_logs(sc)
    sql_sc = SQLContext(sc)

    customSchema = StructType([ \
        StructField("Year", StringType(), True), \
        StructField("Month", FloatType(), True), \
        StructField("DayofMonth", FloatType(), True), \
        StructField("DayOfWeek", FloatType(), True), \
        StructField("DepTime", StringType(), True), \
        StructField("CRSDepTime", StringType(), True), \
        StructField("ArrTime", StringType(), True), \
        StructField("CRSArrTime", StringType(), True), \
        StructField("UniqueCarrier", StringType(), True), \
        StructField("FlightNum", StringType(), True), \
        StructField("TailNum", StringType(), True), \
        StructField("ActualElapsedTime", StringType(), True), \
        StructField("CRSElapsedTime", StringType(), True), \
        StructField("AirTime", StringType(), True), \
        StructField("ArrDelay", StringType(), True), \
        StructField("DepDelay", StringType(), True), \
        StructField("Origin", StringType(), True), \
        StructField("Dest", StringType(), True), \
        StructField("Distance", StringType(), True), \
        StructField("TaxiIn", StringType(), True), \
        StructField("TaxiOut", StringType(), True), \
        StructField("Cancelled", StringType(), True), \
        StructField("CancellationCode", StringType(), True), \
        StructField("Diverted", StringType(), True), \
        StructField("CarrierDelay", StringType(), True), \
        StructField("WeatherDelay", StringType(), True), \
        StructField("NASDelay", StringType(), True), \
        StructField("SecurityDelay", StringType(), True), \
        StructField("LateAircraftDelay", StringType(), True)])

    # read data
    #other options: inferschema='true', header on dataset, other parserLib
    df = sql_sc.read.format('com.databricks.spark.csv').options(header='false',parserLib='univocity').load(input_file, schema=customSchema)
    print("Columns:", df.columns)
    print("Num partitions: %d" % df.rdd.getNumPartitions())
    
    # sanity check data types for scoring code input columns
    # row = df.first()
    # for cell_name in ['Month', 'DayofMonth', 'DayOfWeek']:
    #     cell = row[cell_name] 
    #     print(cell, type(cell))

    # score data
    out_rdd = df.map(lambda x: map_score(x, jmp_model))
    out_df = sql_sc.createDataFrame(out_rdd)
    
    # display or save result
    out_df.describe().show()

    if output_file:
        print("Saving results to %s" % output_file)
        out_df.write.format("com.databricks.spark.csv").option("header", "true").save(output_file)
    
    raw_input("Done. Press Enter to finish...")
    sc.stop()
